
#ifndef __J2K_LIBRARY__
#define __J2K_LIBRARY__

#ifdef J2K_CODEC_EXPORTS

	#define J2K_CODEC_API(ret) extern "C" __declspec(dllexport) ret __stdcall

#else

 #ifdef __cplusplus
	#define J2K_CODEC_API(ret) extern "C" __declspec(dllimport) ret __stdcall
 #else
	#define J2K_CODEC_API(ret) __declspec(dllimport) ret __stdcall 
 #endif

 enum J2K_ErrorList
 {
	 J2KERR_SUCCESS,
	 J2KERR_NOT_ENOUGH_MEMORY,
	 J2KERR_CORRUPTED_DATA,
	 J2KERR_PIPE_FAILURE,
	 J2KERR_INVALID_ARGUMENT,
	 J2KERR_CANCELED,
	 J2KERR_CANT_OPEN_FILE,
	 J2KERR_OPTION_UNRECOGNIZED,
	 J2KERR_NO_SUCH_RESOLUTION,
	 J2KERR_BPP_TOO_SMALL,
	 J2KERR_BUFFER_TOO_SMALL,
	 J2KERR_NOT_PART1_FORMAT,
	 J2KERR_IMAGE_IS_TOO_LARGE,
	 J2KERR_TOO_MANY_RES_LEVEL,
	 J2KERR_TOO_LARGE_CODEBLOCKS,
	 J2KERR_NO_LAZINESS,
	 J2KERR_NO_VCAUSAL,
	 J2KERR_TOO_MANY_COMPONENTS,
	 J2KERR_ONLY_8_BIT_COMPONENTS,
	 J2KERR_ONLY_UNSIG_COMPONENTS,
	 J2KERR_DOWNSAMPLED_COMPONENTS,
	 J2KERR_ROI_NOT_SUPPORTED,
	 J2KERR_PROGR_CHANGE_NOT_SUP,
	 J2KERR_PACKET_HDRS_NOT_SUP,
	 J2KERR_64BIT_BOXES_NOT_SUP,
	 J2KERR_INTERNAL_ERROR
 };

#endif


/*/////////////////////////////////////////////////////////////////////////////////////////
// J2K_getVersion()
//
// Returns codec version and build number in the 0x105678 form, where 10 is a version (1.0)
// and 5678 is a build number.
/////////////////////////////////////////////////////////////////////////////////////////*/

J2K_CODEC_API(int) J2K_getVersion();


/*/////////////////////////////////////////////////////////////////////////////////////////
// J2K_getLastError()
//                                                                                      
// Returns the code of last error. Use J2K_getErrorStr() to get textual description of
// the error.
/////////////////////////////////////////////////////////////////////////////////////////*/

J2K_CODEC_API(int) J2K_getLastError();


/*/////////////////////////////////////////////////////////////////////////////////////////
// J2K_getErrorStr()
//                                                                                      
// Returns the textual description of an error by the error code obtained from getLastError().
// If the errCode is < 0 or > Max number of errors, the "<unknown>" string is returned.
/////////////////////////////////////////////////////////////////////////////////////////*/

J2K_CODEC_API(const char*) J2K_getErrorStr(int errCode);


/*/////////////////////////////////////////////////////////////////////////////////////////
// J2K_Unlock()
//
// This function unlocks the full functionality of J2K-Codec. After you have purchased your
// personal registration key, you need to pass it to this function.
/////////////////////////////////////////////////////////////////////////////////////////*/

J2K_CODEC_API(void) J2K_Unlock(const char *key);


/*/////////////////////////////////////////////////////////////////////////////////////////
// J2K_StartLogging()
//                                                                                      
// This function creates a log-file and starts logging debug messages of J2K-Codec.
// 'level' can be 0 (Normal logging) or 1 (Detailed logging).
// Passing 'append' = 1 allows to append the log session to existing log-file.
// Returns 0 if there was no error.
// NOTES: 
//  1. Log-file name is 'j2k-codec.log'
//  2. The performance will degrade significantly if the logging is on.
/////////////////////////////////////////////////////////////////////////////////////////*/

#define J2K_LOG_NORMAL		0
#define J2K_LOG_DETAILED	1
#define J2K_LOG_ALL		    J2K_LOG_DETAILED

#define J2K_LOG_APPEND		1

J2K_CODEC_API(int) J2K_StartLogging(int level, int append);


/*/////////////////////////////////////////////////////////////////////////////////////////
// J2K_StopLogging()
//                                                                                      
// Use this function to stop logging, initiated by J2K_StartLogging().
/////////////////////////////////////////////////////////////////////////////////////////*/

J2K_CODEC_API(void) J2K_StopLogging();


/*/////////////////////////////////////////////////////////////////////////////////////////
// J2K_Open()
//                                                                                      
// Returns pointer to an internal class, representing the J2K image, or 0 if
// there was an error. Use J2K_getLastError() to obtain the error code.
/////////////////////////////////////////////////////////////////////////////////////////*/

typedef struct
{
	int  (__stdcall *seek)  (void *data_source, int offset);
	int  (__stdcall *read)  (void *ptr, int size, void *data_source);
	void (__stdcall *close) (void *data_source);
}
J2K_Callbacks;

J2K_CODEC_API(void*) J2K_OpenFile(const char *filename);
J2K_CODEC_API(void*) J2K_OpenMemory(unsigned char *src_buffer, int src_size);
J2K_CODEC_API(void*) J2K_OpenCustom(void *data_source, J2K_Callbacks *cbs);

#ifdef __cplusplus
	__inline void* J2K_Open(const char *filename){ return J2K_OpenFile(filename); }
	__inline void* J2K_Open(unsigned char *src_buffer, int src_size){ return J2K_OpenMemory(src_buffer, src_size); }
	__inline void* J2K_Open(void *data_source, J2K_Callbacks *cbs){ return J2K_OpenCustom(data_source, cbs); }
#endif


/*/////////////////////////////////////////////////////////////////////////////////////////
// J2K_GetInfo()
//                                                                                      
// Gets various information about the image.
// Returns J2KERR_SUCCESS or error code if function failed.
/////////////////////////////////////////////////////////////////////////////////////////*/

J2K_CODEC_API(int) J2K_GetInfo(void *image, int *width, int *height, int *components);


/*/////////////////////////////////////////////////////////////////////////////////////////
// J2K_GetResolutions()
//                                                                                      
// Gets the number of available resolution levels.
// Returns J2KERR_SUCCESS or error code if function failed.
/////////////////////////////////////////////////////////////////////////////////////////*/

J2K_CODEC_API(int) J2K_GetResolutions(void *image, int *resolutions);


/*/////////////////////////////////////////////////////////////////////////////////////////
// J2K_GetResolutionDimensions()
//                                                                                      
// Gets the dimensions of a resolution level.
// Returns J2KERR_SUCCESS or error code if function failed.
/////////////////////////////////////////////////////////////////////////////////////////*/

J2K_CODEC_API(int) J2K_GetResolutionDimensions(void *image, int res_level, int *width, int *height);


/*/////////////////////////////////////////////////////////////////////////////////////////
// J2K_Decode()
//                                                                                      
// Decodes the image, previously created with J2K_Open().
// If 'buffer' is null, then the required amount of memory is allocated and its size returned
// through the 'size' argument.
// 'buff_pitch' is a buffer pitch - distance, in bytes, to the start of next line.
/////////////////////////////////////////////////////////////////////////////////////////*/

#ifdef __cplusplus
 J2K_CODEC_API(int) J2K_Decode(void *image, unsigned char **buffer, int *size, char *options=0, int *pitch=0); 
#else
 J2K_CODEC_API(int) J2K_Decode(void *image, unsigned char **buffer, int *size, char *options, int *pitch); 
#endif


/*/////////////////////////////////////////////////////////////////////////////////////////
// J2K_Cancel()
//                                                                                      
// Cancels J2K_Decode() operations.
/////////////////////////////////////////////////////////////////////////////////////////*/

J2K_CODEC_API(void) J2K_Cancel(void *image);


/*/////////////////////////////////////////////////////////////////////////////////////////
// J2K_Close()
//                                                                                      
// Destroys the image, previously created by J2K_Open(). All images must be closed using
// this function to avoid memory leaks.
/////////////////////////////////////////////////////////////////////////////////////////*/

J2K_CODEC_API(void) J2K_Close(void *image);


/*/////////////////////////////////////////////////////////////////////////////////////////
// SHORTCUTS
/////////////////////////////////////////////////////////////////////////////////////////*/

const char *J2K_getLastErrorStr(){ return J2K_getErrorStr(J2K_getLastError()); }


/*/////////////////////////////////////////////////////////////////////////////////////////
// EasyDecode Shortcut
//  Decodes jpeg2000 file into memory buffer.
//  Buffer, image width, height and the number of components are passed back via specified pointers.
//  If succeeded - returns pointer to an internal structure, which represents the image, otherwise - NULL.
//  NOTE! You must not free the buffer yourself - use J2K_Close(j2k_image) instead!
/////////////////////////////////////////////////////////////////////////////////////////*/

#ifndef J2K_CODEC_EXPORTS

/* Internal, undocumented functions. You must never use them.*/
   J2K_CODEC_API(void) J2K_setLastError(int); 
   J2K_CODEC_API(int)  J2K_GetResolutionDimensionsByStr(void *image, char *options, int *width, int *height);

void* J2K_EasyDecode(char *filename, unsigned char **buffer, int *width, int *height, int *components, char *options)
{
 int j2k_err, dummy_size; void *j2k_image=0;
 
 if(!filename || !filename[0] || !buffer || !width || !height || !components){ J2K_setLastError(J2KERR_INVALID_ARGUMENT); return NULL; }

 *buffer=0; /* NOTE: Unlike C++ Wrapper's easyDecode(), which allows to use the user buffer, C EasyDecode() doesn't */

 j2k_image=J2K_OpenFile(filename); if(!j2k_image) return NULL; 

 j2k_err=J2K_GetInfo(j2k_image, width, height, components);
 if(j2k_err==J2KERR_SUCCESS)
 { 
  j2k_err=J2K_GetResolutionDimensionsByStr(j2k_image, options, width, height);
  if(j2k_err==J2KERR_SUCCESS || j2k_err == J2KERR_NO_SUCH_RESOLUTION)
  { 
   j2k_err=J2K_Decode(j2k_image, buffer, &dummy_size, options, NULL); 
  }
 }

 if(j2k_err==J2KERR_SUCCESS) return j2k_image; else { J2K_Close(j2k_image); return NULL; }
}

#endif


/*/////////////////////////////////////////////////////////////////////////////////////////
// C++ Wrapper
/////////////////////////////////////////////////////////////////////////////////////////*/
                                                  

#ifndef J2K_CODEC_EXPORTS

#ifdef __cplusplus

class J2K_Image
{
 protected:

   void *image;

  int  error(int code){ errStr=J2K_getErrorStr(code); return code; }
  void reset(){ image=0; width=height=components=0; errStr=""; buffer=0; size=0; pitch=0; }
  int _open();

 public:

  int width, height, components; const char *errStr;
  unsigned char *buffer; int size, pitch;

 J2K_Image(){ reset(); }

// bool isOK(){ return (image!=NULL); }

 int  open(char *filename){ if(image) close(); image=J2K_Open(filename); return _open(); }
 int  open(unsigned char *src_buffer, int src_size){ if(image) close(); image=J2K_Open(src_buffer, src_size); return _open(); }

 int  decode(char *options=0);

 int  easyDecode(char *filename, char *options=0)
 {
  int j2k_err=open(filename); if(j2k_err==J2KERR_SUCCESS) return decode(options); else return error(j2k_err);
 }

 int  easyDecode(unsigned char *src_buffer, int src_size, char *options=0)
 {
  int j2k_err=open(src_buffer, src_size); if(j2k_err==J2KERR_SUCCESS) return decode(options); else return error(j2k_err);
 }

 void close(){ J2K_Close(image); reset(); }

~J2K_Image(){ close(); }
};

//--------------------------------------------------------------------------------------

int J2K_Image::_open()
{
 if(!image) return J2K_getLastError();

 int j2k_err=J2K_GetInfo(image, &width, &height, &components); 
 if(j2k_err!=J2KERR_SUCCESS){ close(); return j2k_err; } else return J2KERR_SUCCESS;
}
//--------------------------------------------------------------------------------------

int J2K_Image::decode(char *options)
{
 if(!image) return error(J2KERR_SUCCESS); // Do nothing if there's nothing to do
  
 int j2k_err=J2K_GetResolutionDimensionsByStr(image, options, &width, &height);
 if(j2k_err==J2KERR_SUCCESS || j2k_err==J2KERR_NO_SUCH_RESOLUTION)
 {
  j2k_err=J2K_Decode(image, &buffer, &size, options, &pitch);
  if(j2k_err==J2KERR_SUCCESS) return J2KERR_SUCCESS;
 }

 close(); return error(j2k_err);
}

#endif

#endif

/* ///////////////////////////////////////////////////////////////////////////////////// */

#endif
